import os
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.template.utils import InvalidTemplateEngineError

from appconf import AppConf


class CompressorConf(AppConf):
    # Main switch
    ENABLED = not settings.DEBUG
    # Allows changing verbosity from the settings.
    VERBOSE = False
    # GET variable that disables compressor e.g. "nocompress"
    DEBUG_TOGGLE = None
    # the backend to use when parsing the JavaScript or Stylesheet files
    PARSER = "compressor.parser.AutoSelectParser"
    OUTPUT_DIR = "CACHE"
    STORAGE_ALIAS = "compressor"
    STORAGE = "compressor.storage.CompressorFileStorage"

    COMPRESSORS = dict(
        css="compressor.css.CssCompressor",
        js="compressor.js.JsCompressor",
    )

    URL = None
    ROOT = None

    FILTERS = {
        "css": [
            "compressor.filters.css_default.CssAbsoluteFilter",
            "compressor.filters.cssmin.rCSSMinFilter",
        ],
        "js": ["compressor.filters.jsmin.rJSMinFilter"],
    }

    CSS_HASHING_METHOD = "mtime"

    PRECOMPILERS = (
        # ('text/coffeescript', 'coffee --compile --stdio'),
        # ('text/less', 'lessc {infile} {outfile}'),
        # ('text/x-sass', 'sass {infile} {outfile}'),
        # ('text/stylus', 'stylus < {infile} > {outfile}'),
        # ('text/x-scss', 'sass --scss {infile} {outfile}'),
    )
    CACHEABLE_PRECOMPILERS = ()
    CLOSURE_COMPILER_BINARY = "java -jar compiler.jar"
    CLOSURE_COMPILER_ARGUMENTS = ""
    YUI_BINARY = "java -jar yuicompressor.jar"
    YUI_CSS_ARGUMENTS = ""
    YUI_JS_ARGUMENTS = ""
    YUGLIFY_BINARY = "yuglify"
    YUGLIFY_CSS_ARGUMENTS = "--terminal"
    YUGLIFY_JS_ARGUMENTS = "--terminal"
    CLEAN_CSS_BINARY = "cleancss"
    CLEAN_CSS_ARGUMENTS = ""
    DATA_URI_MAX_SIZE = 1024

    # the cache backend to use
    CACHE_BACKEND = None
    # the dotted path to the function that creates the cache key
    CACHE_KEY_FUNCTION = "compressor.cache.simple_cachekey"
    # rebuilds the cache every 30 days if nothing has changed.
    REBUILD_TIMEOUT = 60 * 60 * 24 * 30  # 30 days
    # the upper bound on how long any compression should take to be generated
    # (used against dog piling, should be a lot smaller than REBUILD_TIMEOUT
    MINT_DELAY = 30  # seconds
    # check for file changes only after a delay
    MTIME_DELAY = 10  # seconds
    # enables the offline cache -- also filled by the compress command
    OFFLINE = False
    # invalidates the offline cache after one year
    OFFLINE_TIMEOUT = 60 * 60 * 24 * 365  # 1 year
    # The context to be used when compressing the files "offline"
    OFFLINE_CONTEXT = {}
    # The name of the manifest file (e.g. filename.ext)
    OFFLINE_MANIFEST = "manifest.json"
    OFFLINE_MANIFEST_STORAGE_ALIAS = "compressor-offine"
    OFFLINE_MANIFEST_STORAGE = "compressor.storage.OfflineManifestFileStorage"
    # The Context to be used when TemplateFilter is used
    TEMPLATE_FILTER_CONTEXT = {}
    # Placeholder to be used instead of settings.COMPRESS_URL during offline compression.
    # Affects manifest file contents only.
    URL_PLACEHOLDER = "/__compressor_url_placeholder__/"

    # Returns the Jinja2 environment to use in offline compression.
    def JINJA2_GET_ENVIRONMENT():
        alias = "jinja2"
        try:
            from django.template import engines

            return engines[alias].env
        except InvalidTemplateEngineError:
            raise InvalidTemplateEngineError(
                "Could not find config for '{}' "
                "in settings.TEMPLATES. "
                "COMPRESS_JINJA2_GET_ENVIRONMENT() may "
                "need to be defined in settings".format(alias)
            )
        except ImportError:
            return None

    class Meta:
        prefix = "compress"

    def configure_root(self, value):
        # Uses Django's STATIC_ROOT by default
        if value is None:
            value = settings.STATIC_ROOT
        if value is None:
            raise ImproperlyConfigured(
                "COMPRESS_ROOT defaults to " + "STATIC_ROOT, please define either"
            )
        return os.path.normcase(os.path.abspath(value))

    def configure_url(self, value):
        # Uses Django's STATIC_URL by default
        if value is None:
            value = settings.STATIC_URL
        if not value.endswith("/"):
            raise ImproperlyConfigured(
                "URL settings (e.g. COMPRESS_URL) " "must have a trailing slash"
            )
        return value

    def configure_cache_backend(self, value):
        if value is None:
            value = "default"
        return value

    def configure_offline_context(self, value):
        if not value:
            value = {"STATIC_URL": settings.STATIC_URL}
        return value

    def configure_template_filter_context(self, value):
        if not value:
            value = {"STATIC_URL": settings.STATIC_URL}
        return value

    def configure_precompilers(self, value):
        if not isinstance(value, (list, tuple)):
            raise ImproperlyConfigured(
                "The COMPRESS_PRECOMPILERS setting "
                "must be a list or tuple. Check for "
                "missing commas."
            )
        return value



class ParserBase:
    """
    Base parser to be subclassed when creating an own parser.
    """

    def __init__(self, content):
        self.content = content

    def css_elems(self):
        """
        Return an iterable containing the css elements to handle
        """
        raise NotImplementedError

    def js_elems(self):
        """
        Return an iterable containing the js elements to handle
        """
        raise NotImplementedError

    def elem_attribs(self, elem):
        """
        Return the dictionary like attribute store of the given element
        """
        raise NotImplementedError

    def elem_content(self, elem):
        """
        Return the content of the given element
        """
        raise NotImplementedError

    def elem_name(self, elem):
        """
        Return the name of the given element
        """
        raise NotImplementedError

    def elem_str(self, elem):
        """
        Return the string representation of the given elem
        """
        raise NotImplementedError



#!/usr/bin/env python
import optparse
import sys


def main():
    p = optparse.OptionParser()
    p.add_option(
        "-f",
        "--file",
        action="store",
        type="string",
        dest="filename",
        help="File to read from, defaults to stdin",
        default=None,
    )
    p.add_option(
        "-o",
        "--output",
        action="store",
        type="string",
        dest="outfile",
        help="File to write to, defaults to stdout",
        default=None,
    )

    options, arguments = p.parse_args()

    if options.filename:
        with open(options.filename) as f:
            content = f.read()
    else:
        content = sys.stdin.read()

    content = content.replace("background:", "color:")

    if options.outfile:
        with open(options.outfile, "w") as f:
            f.write(content)
    else:
        print(content)


if __name__ == "__main__":
    main()



from compressor.base import Compressor, SOURCE_HUNK, SOURCE_FILE
from compressor.conf import settings


class CssCompressor(Compressor):

    output_mimetypes = {"text/css"}

    def split_contents(self):
        if self.split_content:
            return self.split_content
        self.media_nodes = []
        for elem in self.parser.css_elems():
            data = None
            elem_name = self.parser.elem_name(elem)
            elem_attribs = self.parser.elem_attribs(elem)
            if (
                elem_name == "link"
                and "rel" in elem_attribs
                and elem_attribs["rel"].lower() == "stylesheet"
            ):
                basename = self.get_basename(elem_attribs["href"])
                filename = self.get_filename(basename)
                data = (SOURCE_FILE, filename, basename, elem)
            elif elem_name == "style":
                data = (SOURCE_HUNK, self.parser.elem_content(elem), None, elem)
            if data:
                self.split_content.append(data)
                media = elem_attribs.get("media", None)
                # Append to the previous node if it had the same media type
                append_to_previous = (
                    self.media_nodes and self.media_nodes[-1][0] == media
                )
                # and we are not just precompiling, otherwise create a new node.
                if append_to_previous and settings.COMPRESS_ENABLED:
                    self.media_nodes[-1][1].split_content.append(data)
                else:
                    node = self.copy(content=self.parser.elem_str(elem))
                    node.split_content.append(data)
                    self.media_nodes.append((media, node))
        return self.split_content

    def output(self, *args, **kwargs):
        if (
            settings.COMPRESS_ENABLED
            or settings.COMPRESS_PRECOMPILERS
            or kwargs.get("forced", False)
        ):
            # Populate self.split_content
            self.split_contents()
            if hasattr(self, "media_nodes"):
                ret = []
                for media, subnode in self.media_nodes:
                    subnode.extra_context.update({"media": media})
                    ret.append(subnode.output(*args, **kwargs))
                return "".join(ret)
        return super().output(*args, **kwargs)



import os

TEST_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), "tests")


CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "unique-snowflake",
    }
}

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

INSTALLED_APPS = [
    "django.contrib.staticfiles",
    "compressor",
    "sekizai",
]

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
    "compressor.finders.CompressorFinder",
]

STATIC_URL = "/static/"


STATIC_ROOT = os.path.join(TEST_DIR, "static")

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "DIRS": [
            # Specifically choose a name that will not be considered
            # by app_directories loader, to make sure each test uses
            # a specific template without considering the others.
            os.path.join(TEST_DIR, "test_templates"),
        ],
    },
    {
        "BACKEND": "django.template.backends.jinja2.Jinja2",
        "APP_DIRS": True,
        "DIRS": [
            # Specifically choose a name that will not be considered
            # by app_directories loader, to make sure each test uses
            # a specific template without considering the others.
            os.path.join(TEST_DIR, "test_templates_jinja2"),
        ],
    },
]

SECRET_KEY = "iufoj=mibkpdz*%bob952x(%49rqgv8gg45k36kjcg76&-y5=!"

PASSWORD_HASHERS = ("django.contrib.auth.hashers.UnsaltedMD5PasswordHasher",)

MIDDLEWARE_CLASSES = []

USE_TZ = True
